from pathlib import Path

import matplotlib
import matplotlib.pyplot as plt
import pandas as pd
from astropy.io import fits
from tqdm.auto import tqdm

from .akatsuki.ir2 import IR2FileName

matplotlib.use("agg")


def create_preview(path):
    data, h = fits.getdata(path, header=True)
    fn = IR2FileName(path.name)
    fig, ax = plt.subplots()
    ax.imshow(data, vmin=0, cmap="gray", origin="lower")
    savename = path.with_suffix(".png")
    ax.set_title(f"{fn.datetime}, exp: {h['EXPOSURE']}, filter: {fn.wavelength}")
    fig.savefig(savename, dpi=150)
    plt.close(fig)


def create_browse_images(filelist):
    for p in tqdm(filelist.full_path):
        try:
            create_preview(p)
        except ValueError:
            print(f"problem with {p.name}")


def find_best_header(path):
    hdul = fits.open(path)
    best_hdu = None
    while hdul:
        hdu = hdul.pop(0)
        best_hdu = hdu
        if isinstance(hdu, fits.hdu.image.ImageHDU):
            break
    return best_hdu.header


def convert_header_to_dataframe(header, index=None):
    headerdict = dict(header)
    # there's a huge empty string at the end of headers
    # if it's called "", then it's removed, otherwise no harm done.
    _ = headerdict.pop("", None)
    # remove COMMENT and ORIGIN
    keys_to_delete = ["COMMENT", "ORIGIN"]
    for key in keys_to_delete:
        _ = headerdict.pop(key, None)
    index = pd.Index([index], name="filename")
    return pd.DataFrame(pd.Series(headerdict).to_dict(), index=index)


def write_out_fits_headers(folder):
    """Writing out FITS headers into CSV.

    This function will search for FITS files in the given FOLDER.
    It then scans each FITS file for ImageHDUs, and if not there, will take any HDU's header.
    It collects all header data, removes the keys "COMMENT", "HISTORY", and "ORIGIN",
    and combines everything into a CSV file.

    Parameters
    ----------
    folder : str, pathlib.Path
        Folder in which to look for FITS files. Can be ".".

    Returns
    -------
    Nothing, but creates a CSV file named: "<FOLDER>_fits_headers.csv"
    """
    if folder is None:
        folder = Path(".").absolute()
    folder = Path(folder).absolute()
    fits_files = folder.glob("*.fits")

    bucket = []

    for f in fits_files:
        header = find_best_header(f)
        bucket.append(convert_header_to_dataframe(header, index=f.name))
    df = pd.concat(bucket)
    savename = f"{folder.name}_fits_headers.csv"
    savepath = folder / savename
    df.to_csv(savepath)



import astropy.io.fits as pf
import numpy as np
from astropy.convolution import Gaussian2DKernel, interpolate_replace_nans
from matplotlib.pyplot import *
from scipy import signal
from tqdm import tqdm_notebook as tqdm

import cv2

# import warnings
# warnings.filterwarnings("ignore")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `venim` package."""

from pathlib import Path
from urllib.request import urlretrieve

import pytest
from astropy.io import fits

# from click.testing import CliRunner
# from venim import cli, stats

server_url = "https://atmos.nmsu.edu/PDS/data/"
testfile_url = server_url + "vcoir2_0001/data/r0009/ir2_20160313_075709_174_l1b_v10.fit"


@pytest.fixture(scope="session")
def image_file(tmpdir_factory):
    fn = tmpdir_factory.mktemp("data").join("testimg.fits")
    print("URL", testfile_url)
    print("FN", str(fn))
    urlretrieve(str(testfile_url), str(fn))
    return fn


# @pytest.fixture
# def response():
#     """Sample pytest fixture.

#     See more at: http://doc.pytest.org/en/latest/fixture.html
#     """
#     # import requests
#     # return requests.get('https://github.com/michaelaye/cookiecutter-pypackage-conda')


# def test_content(response):
#     """Sample pytest test function with the pytest fixture as an argument."""
#     # from bs4 import BeautifulSoup
#     # assert 'GitHub' in BeautifulSoup(response.content).title.string


# def test_command_line_interface():
#     """Test the CLI."""
#     runner = CliRunner()
#     result = runner.invoke(cli.main)
#     assert result.exit_code == 0
#     assert 'venim.cli.main' in result.output
#     help_result = runner.invoke(cli.main, ['--help'])
#     assert help_result.exit_code == 0
#     assert '--help  Show this message and exit.' in help_result.output



# -*- coding: utf-8 -*-
"""Akatsuki IR2 utilities.

Utilities to download and manage Akatsuki IR2 data from NASA's PDS.
"""
import webbrowser
from pathlib import Path
from urllib.request import urlretrieve

import pandas as pd
import pvl
import requests
from astropy.io import fits
from tqdm.auto import tqdm
from urlpath import URL

from ..config import config
from ..image import Image
from ..pathmanager import PathManager

storage_root = Path(config["venim_path"]).expanduser()
storage_root.mkdir(exist_ok=True, parents=True)

level_urls = {
    "raw": URL("https://atmos.nmsu.edu/PDS/data/vcoir2_0001/data/"),
    "calibrated": URL("https://atmos.nmsu.edu/PDS/data/vcoir2_1001/data/l2b/"),
    "geom": URL("https://atmos.nmsu.edu/PDS/data/vcoir2_2001/geometry/"),
}

doc_urls = {
    "imagecoord": URL(
        "https://atmos.nmsu.edu/PDS/data/vcoir2_1001/document/imagecoord.txt"
    ),
    "fits_dic": URL(
        "https://atmos.nmsu.edu/PDS/data/vcoir2_1001/document/vco_fits_dic_v05.html"
    ),
    "ir2cal": URL("https://atmos.nmsu.edu/PDS/data/vcoir2_1001/document/ir2cal.txt"),
    "onlabels": URL(
        "https://atmos.nmsu.edu/PDS/data/vcoir2_1001/document/onlabels.txt"
    ),
    "vco_observation_program": URL(
        "https://atmos.nmsu.edu/PDS/data/vcoir2_1001/document/vco_obsprg_v04.html"
    ),
}

HEADER_KEYWORDS = ["EXPOSURE", "NAXIS1", "NAXIS2", "I2_T_C1", "I2_T_C2", "I2_T_OP"]


def get_rev_filelist(rev, level="calibrated"):
    """Scrape PDS file listing using pandas.read_html.

    First scrape, using `pd.read_html`, and then add columns `datestr`, `timestr`, and
    `datetime`.

    Parameters
    ----------
    rev : str
        Revolution (i.e. orbit) string in format `rxxxx`
    level : {'raw', 'calibrated','geom'}, optional.
        Data level to parse directory for. Default: 'calibrated'

    Returns
    -------
    pd.DataFrame
        DataFrame with the content of the parsed file listing.
    """
    results = pd.read_html(str(level_urls[level] / rev), skiprows=2)
    df = results[0]
    df = df.drop("Unnamed: 0", axis=1).dropna(how="all")
    df.columns = ["filename", "time", "size", "unnamed"]
    df = df[["filename", "time", "size"]]
    df["datestr"] = df.filename.map(lambda x: x.split("_")[1])
    df["timestr"] = df.filename.map(lambda x: x.split("_")[2])
    df["datetime"] = pd.to_datetime(df.datestr + " " + df.timestr)
    return df


def download_rev(rev, level="calibrated", force=False):
    """Download data for a given revolution (i.e. orbit)

    Will check for existence of files and not download again.
    Existing files can be overwritten using force=True as a parameter.

    Parameters
    ----------
    rev : str
        Revolution ID, i.e. "r0025"
    level : {"raw", "calibrated", "geom}, optional.
        String identifying the data level wanted. Default: "calibrated".
    force : bool, optional
        Overwrite existing data.
    """
    dirlist = get_rev_filelist(rev)
    storage_dir = Path(config["venim_path"]).expanduser()
    storage_dir.mkdir(exist_ok=True, parents=True)
    for fname in dirlist.filename:
        print(f"Downloading {fname}")
        url = level_urls[level] / rev / fname
        print(url)
        urlretrieve(str(url), storage_dir, use_tqdm=False)


def _download(url, savepath):
    # urlretrieve(str(url), savepath)
    r = requests.get(str(url), stream=True)
    with open(savepath, "wb") as fd:
        for chunk in r.iter_content(chunk_size=128):
            fd.write(chunk)


def download_list_of_urls(savedir, urllist, force=False, test=False, verbose=False):
    savedir.mkdir(exist_ok=True, parents=True)
    if test:
        print("The following files would be downloaded:")
    for url in tqdm(urllist):
        savepath = savedir / url.name
        if savepath.exists() and not force:
            continue
        if verbose:
            print(f"Downloading\n{url}")
            print(f"to\n{savepath}")
        if not test:
            _download(url, savepath)


class Downloader:
    def __init__(self, level="calibrated", orbit=None):
        """Downloader class for Akatsuki IR2 data.

        The orbit can be set at creation of a Downloader object, or set later.

        Examples
        --------
        >>> dl = Downloader(orbit=25)
        >>> dl.orbit
        25

        or

        >>> dl = Dowloader()
        >>> dl.orbit = 25
        >>> dl.orbit
        25

        Parameters
        ----------
        level : {'raw','calibrated','geom}
            Akatsuki data kind to work with
        orbit : str, int
            The Akatsuki orbit. Both an integer and the revolution string 'rxxxx' is allowed.
        """
        self._level = level
        self.orbit = orbit

    @property
    def orbit(self):
        """str: Akatsuki's revolution string with format 'rxxxx'.

        The setter method takes both the int and str version as new value.
        """
        return self._orbit

    @orbit.setter
    def orbit(self, new_orbit):
        if new_orbit is None:
            self._orbit = None
            return
        if not str(new_orbit).startswith("r"):
            new_orbit = "r" + str(new_orbit).zfill(4)
        self._orbit = new_orbit

    @property
    def level(self):
        return [key for key in level_urls.keys() if key.startswith(self._level)][0]

    @level.setter
    def level(self, new_level):
        try:
            new_level = [key for key in level_urls.keys() if key.startswith(new_level)][
                0
            ]
        except IndexError:
            raise ValueError(
                f"New level name '{new_level}' not supported. Needs to be one of {list(level_urls.keys())}."
            )
        self._level = new_level

    def get_orbit_filelist(self, orbit=None):
        if orbit is None:
            orbit = self.orbit
        else:
            self.orbit = orbit
        df = get_rev_filelist(self.orbit, self.level)
        self.filelist = df
        return df

    @property
    def savedir(self):
        pm = IR2PathManager()
        return pm.instr_savedir / self.level

    def get_urls_for_orbit(self, orbit=None):
        if orbit is None:
            orbit = self.orbit
        else:
            self.orbit = orbit
        df = self.get_orbit_filelist(self.orbit)
        urls = [
            level_urls[self.level] / self.orbit / filename for filename in df.filename
        ]
        return pd.Series(urls)

    def download_orbit_files(self, orbit=None, only_fits=True, **kwargs):
        if orbit is None:
            orbit = self.orbit
        else:
            self.orbit = orbit
        urls = self.get_urls_for_orbit(orbit)
        if only_fits:
            urls = [url for url in urls if url.suffix.endswith(".fit")]
        orbit_savedir = self.savedir / self.orbit
        download_list_of_urls(orbit_savedir, urls, **kwargs)


class IR2FileName:
    """Class to dismantle an IR2FileName.

    Each splittable part of the filename is reachable as a class attribute, and the
    datestr and timestr data has been used to create a `pandas.Timestamp`, which can
    easily be converted back to a datetime object.

    Example
    -------
    >>> irfn = IR2FileName('ir2_20160829_201008_202_l2b_v10.fit')
    >>> irfn.wavelength
    202
    >>> irfn.datetime
    Timestamp('2016-08-29 20:10:08')

    Parameters
    ----------
    fname : str
        Akatsuki filename in the format `ir2_20160829_201008_202_l2b_v10.fit`
    """

    def __init__(self, fname):
        self.name = Path(fname).name

    @property
    def tokens(self):
        "list: Parts of the filename, separated by '_'."
        return self.name.split("_")

    @property
    def instr(self):
        "str: Instrument ID."
        return self.tokens[0]

    @property
    def datestr(self):
        "str: The date string of the filename."
        return self.tokens[1]

    @property
    def date(self):
        "pd.Timestamp: Date string converted to proper time object."
        return pd.to_datetime(self.datestr)

    @property
    def timestr(self):
        return self.tokens[2]

    @property
    def time(self):
        return pd.to_datetime(self.timestr)

    @property
    def datetime(self):
        return pd.to_datetime(self.datestr + self.timestr)

    @property
    def wavelength(self):
        return self.tokens[3]

    @property
    def in_micron(self):
        "float: Filter wavelength converted to micron."
        return int(self.wavelength) / 100

    @property
    def level(self):
        return self.tokens[4]

    @property
    def version(self):
        return self.tokens[5]

    def __str__(self):
        s = f"IR2 FileName {self.name}\n"
        s += f"DateTime: {self.datetime}\n"
        s += f"Wavelength: {self.in_micron} micron.\n"
        s += f"Level: {self.level}\n"
        s += f"Version: {self.version}\n"
        return s

    def __repr__(self):
        return self.__str__()


class IR2PathManager(PathManager):
    """Class to manage files for Akatsuki IR2 data.

    Offers method `list_files_for_orbit` to get a list of found files per orbit in
    form of a pandas.DataFrame.
    Parameters
    ----------
    level : {'raw', 'calibrated', 'geom'}
        Which IR2 data to choose from.
    orbit : int, str; optional
        Akatsuki orbit, both revolution string rxxxx and integer number is allowed.
    full_paths : bool, optional
        Switch to show full paths on disk.
    """

    def __init__(self, level="calibrated", orbit=None, full_paths=False):
        super().__init__("akatsuki", "ir2")
        self.level = level
        self.orbit = orbit
        self.full_paths = full_paths

    @property
    def orbit(self):
        """str: Akatsuki orbit in format of revolution string 'rxxxx'."""
        return self._orbit

    @orbit.setter
    def orbit(self, new_orbit):
        if new_orbit is None:
            self._orbit = None
            return
        if not str(new_orbit).startswith("r"):
            new_orbit = "r" + str(new_orbit).zfill(4)
        self._orbit = new_orbit

    def list_files_for_orbit(self, orbit=None):
        """Get a file listing as a pandas.DataFrame.

        Parameters
        ----------
        orbit : str, int
            Akatsuki orbit, either as rxxxx string or integer.
        full_paths : bool
            Switch to show either truncated or pull paths.

        Returns
        -------
        pandas.DataFrame
            The Dataframe lists filenames and the datetimes for the files.
        """
        if orbit is None:
            orbit = self.orbit
        pathlist = list((self.savedir).glob("*.fit*"))
        namelist = [f.name for f in pathlist]
        df = pd.DataFrame({"filename": namelist, "full_path": pathlist})
        df["datetime"] = df.filename.map(lambda x: IR2FileName(x).datetime)
        df["wavelength"] = df.filename.map(lambda x: int(IR2FileName(x).wavelength))
        df = pd.concat([df, df.apply(get_header_keywords, axis=1)], axis=1)
        # for keyword in HEADER_KEYWORDS:
        #     df[keyword] = df.full_path.map(lambda x: fits.open(x)[1].header[keyword])
        columns = list(df.columns)
        columns.remove("full_path")
        columns.append("full_path")
        for col in "naxis1 naxis2".upper().split():
            df[col] = df[col].astype("int")
        return df[columns].set_index("datetime")

    def get_path(self, index):
        ""
        filelist = self.list_files_for_orbit(full_paths=True)
        return filelist.filename.iloc[index]

    def get_label(self, index):
        p = self.get_path(index)
        return pvl.load(str(p.with_suffix(".lbl")))

    @property
    def savedir(self):
        try:
            return self.instr_savedir / self.level / self.orbit
        except TypeError:
            raise ValueError("Orbit must be set first.")


# helper functions
def get_header_keywords(row):
    header = fits.open(row.full_path)[1].header
    d = {}
    for kw in HEADER_KEYWORDS:
        d[kw] = header[kw]
    return pd.Series(d)


def get_exposure(row):
    fits.open(row.full_path)[1].header[""]


def get_orbit_file_list(orbit):
    "Shortcut to simply get the list of files from IR2PathManager."
    pm = IR2PathManager(orbit=orbit)
    return pm.list_files_for_orbit()


def get_file_path(id):
    """
    Glob all IR2 data for given file name.

    Parameters
    ----------
    id : str
        IR2 filename

    Returns
    -------
    pathlib.Path
        Local path to filename given.
    """
    pm = IR2PathManager(orbit=0)
    return list(pm.savedir.parent.glob(f"*/{id}"))[0]


def get_file_header(id):
    """
    Get header for given file name.

    Searches all IR2 data for given filename and returns header of found file.

    Parameters
    ----------
    id : str
        IR2 filename.

    Returns
    -------
    fits.Header
        FITS ImageHDU header for given filename.
    """
    p = get_file_path(id)
    return fits.open(p)[1].header


def get_file_data(id):
    """
    Get data for given file name.

    Searches all IR2 data for given filename and returns numpy data for found file.

    Parameters
    ----------
    id: str
        IR2 filename.

    Returns
    -------
    numpy.array
        Numpy 2D array of ImageHDU of the FITS file for given filename.
    """
    p = get_file_path(id)
    return fits.getdata(p)


def getdata(id, header=False):
    """
    Simulate fits.getdata, for just a filename without full path.
    Will search all IR2 data for given file name.

    Parameters
    ----------
    id : str
        File name
    header : bool, optional
        Boolean switch to control if you also want the header. Default: False

    Returns
    -------
    numpy.array(, fits.Header)
        Return numpy.array (for ImageHDU), and optionally also the fits.Header
    """
    p = Path(id)
    if not p.is_absolute():
        p = get_file_path(id)
    return fits.getdata(p, header=header)


def list_available_doc_urls():
    "List the available URLs to IR2 documentation on PDS."
    for k, v in doc_urls.items():
        print(k)
        print(v)


def open_doc_url(doc_key):
    webbrowser.open_new(str(doc_urls[doc_key]))


def open_fits_dic():
    "Open new browser tab for FITS DIC."
    open_doc_url("fits_dic")


class IR2Image(Image):
    def __init__(self, p_or_id):
        p = Path(p_or_id)
        if not p.is_absolute():
            path = get_file_path(p_or_id)
        else:
            path = p_or_id
        super().__init__(path)

    @property
    def wavelength(self):
        return IR2FileName(self.name).wavelength

    @property
    def exposure(self):
        return self.header["EXPOSURE"]

    @property
    def imagetime(self):
        return IR2FileName(self.name).datetime



from pathlib import Path

import toml


def set_venim_storage_path():
    path = input("Provide path where VENIM data should be stored:")
    config = {}
    config["venim_path"] = path
    with configpath.open("w") as f:
        toml.dump(config, f)


configpath = Path("~/.venim_config.toml").expanduser()

if not configpath.exists():
    set_venim_storage_path()

with configpath.open() as f:
    config = toml.load(f)



from pathlib import Path

import astropy.io.fits as pf
import pandas as pd
from tqdm.auto import tqdm

irtf_translator = {
    "name": "IRAFNAME",
    "date": "DATE_OBS",
    "time": "TIME_OBS",
}


def scan_image_directory(path):
    """Scan directory of FITS files to create basic stats.

    Creates CSV file ready to be read by pandas and print-out of the stats if
    less than 100 entries.

    Parameters
    ----------
    path : str, pathlib.Path

    Returns
    -------
    pd.DataFrame
        DataFrame containing the collected stats
    """
    input_dir = Path(path)
    directory_file = input_dir / "directory.csv"
    files = list(input_dir.glob("*.fits"))

    print(len(files), "images found.")

    # fields = [
    #     "name",
    #     "date",
    #     "time",
    #     "filter",
    #     "exposure",
    #     "hasSlit",
    #     "isValid",
    #     "angle",
    #     "radius",
    #     "area",
    #     "centerY",
    #     "centerX",
    #     "sublon",
    #     "sublat",
    #     "isFull",
    #     "group",
    # ]

    bucket = []

    print("Scanning directory...")
    for fitspath in tqdm(files):
        _, head = pf.getdata(fitspath, header=True)
        line = {}
        line["name"] = fitspath.name
        line["date"] = head["DATE_OBS"]
        line["time"] = head["TIME_OBS"]
        line["filter"] = head["GFLT"]
        line["exposure"] = head["ELAPTIME"]
        line["hasSlit"] = head["SLIT"] != "Mirror"
        line["isValid"] = head["ELAPTIME"] == 0.482500 and head["GFLT"] == "contK"
        bucket.append(line)

    df = pd.DataFrame(bucket)
    df.to_csv(directory_file)
    print("Metadata CSV generated at", directory_file)
    return df



# -*- coding: utf-8 -*-
"""Tools to work with images.
"""
import circle_fit as cf
import holoviews as hv
import numpy as np
import pandas as pd
from astropy.io import fits
from holoviews import opts
from skimage.exposure import equalize_adapthist as equalize
from skimage.exposure import rescale_intensity

hv.extension("bokeh")
opts.defaults(
    opts.Image(tools=["hover"], cmap="gray"),
    opts.Points(color="red", marker="x", size=20),
)


class Image:
    def __init__(self, path):
        self.path = path
        self.data, self.header = fits.getdata(path, header=True)

    @property
    def name(self):
        return self.path.name

    @property
    def full_frame(self):
        newimg = np.zeros((1024, 1024))
        ll_row = self.header["P_POSLLY"] - 1
        ll_col = self.header["P_POSLLX"] - 1
        ur_row = self.header["P_POSURY"]
        ur_col = self.header["P_POSURX"]
        newimg[ll_row:ur_row, ll_col:ur_col] = self.data
        return newimg

    @property
    def rescaled(self):
        return rescale_intensity(self.full_frame, out_range="float")

    @property
    def equalized(self):
        return equalize(self.rescaled)

    def plot(self, sqrt=False, pmin=1, pmax=99, with_fit=False):
        data = self.full_frame
        data[data < 0] = np.nan
        if sqrt:
            data = np.sqrt(data)
        vmin, vmax = np.percentile(data[~np.isnan(data)], (pmin, pmax))
        if sqrt:
            data = np.sqrt(data)
        img = hv.Image(
            (np.arange(1024), np.arange(1024), data),
            label=f"{self.plot_title}, VRange: {pmin}-{pmax} %",
        ).opts(clim=(vmin, vmax), frame_height=500, frame_width=500,)
        if with_fit:
            return img * self.circle_fit
        else:
            return img

    @property
    def plot_title(self):
        t = f"{self.imagetime}, Filter: {self.wavelength}, Exp: {self.exposure} s"
        return t

    def plot_equalized(self):
        return hv.Raster(self.equalized).redim.range(z=(0, None))

    def annotate(self):
        points = hv.Points([]).opts(width=500, height=500, padding=0, responsive=False)
        self.annotator = hv.annotate.instance()
        layout = self.annotator(
            self.plot().opts(frame_height=400, frame_width=400,) * points,
            name="Limb Points",
        )
        return layout

    @property
    def circle_fit(self):
        xCtr, yCtr, r, _ = cf.least_squares_circle(self.points_data.values)
        return hv.Ellipse(xCtr, yCtr, 2 * r).opts(color="red")

    @property
    def points_data(self):
        try:
            return self.annotator.annotated.dframe()
        except AttributeError:
            return pd.DataFrame({"x": [], "y": []})

    @property
    def points_path(self):
        return self.path.with_suffix(".csv")

    def store_points(self):
        self.points_data.to_csv(self.points_path, index=False)

    def get_stored_points(self):
        return pd.read_csv(self.points_path)



from pathlib import Path

from .config import config

storage_root = Path(config["venim_path"]).expanduser()


class PathManager:
    def __init__(self, mission, instr):
        self.mission = mission
        self.instr = instr

    @property
    def instr_savedir(self):
        return storage_root / self.mission / self.instr



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from os import path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))


with open(path.join(here, "README.rst"), encoding="utf-8") as readme_file:
    readme = readme_file.read()

with open(path.join(here, "HISTORY.rst"), encoding="utf-8") as history_file:
    history = history_file.read()

setup(
    name="venim",
    version="0.9.2",
    description="Python tools for Venus Image Analysis",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/x-rst",
    author="K.-Michael Aye",
    author_email="kmichael.aye@gmail.com",
    url="https://github.com/michaelaye/venim",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.6, <4",
    entry_points={"console_scripts": ["write_fits_headers=venim.fits_cli:main"]},
    include_package_data=True,
    install_requires=[
        "numpy",
        "pandas",
        "matplotlib",
        "planetarypy",
        "urlpath",
        "astropy",
        "holoviews",
        "circle-fit",
        "fire",
    ],
    license="MIT license",
    # zip_safe=False,
    keywords="venus images pds data",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
    ],
)



import numpy as np
import cv2


def loadMask(path):
    img = cv2.imread(path, 0)
    print(img)
    mask = img != 0
    return mask + 0


def crescentMask(center, area, outer_radius, pixel_angle, border):

    # Minimum inner pixel radius of the crescent
    min_inner_radius = (1 - 2 * area) * outer_radius

    # Define indices relative to center
    y, x = np.indices((512, 512))
    y = y - center[0]
    x = x - center[1]

    # Define angle array
    angle = np.arctan2(y, x)
    angle -= np.radians(pixel_angle)
    angle[angle < -np.pi / 2] += 2 * np.pi

    # Define distance array
    dist = np.sqrt(x ** 2 + y ** 2)

    # Define crescent relative to angle and dist arrays
    crescent = (
        (angle > 0)
        & (angle < np.pi)
        & (
            dist
            > (
                (min_inner_radius * outer_radius)
                / np.sqrt(
                    (min_inner_radius * np.cos(angle)) ** 2
                    + (outer_radius * np.sin(angle)) ** 2
                )
            )
        )
        & (dist < outer_radius)
    )

    # Add a border around the mask to show more
    if border > 0:
        y, x = np.indices((border * 2 - 1, border * 2 - 1))
        kernel = np.array(
            np.sqrt((y - border + 1) ** 2 + (x - border + 1) ** 2) < border,
            dtype=np.uint8,
        )

        crescent = cv2.dilate(np.array(crescent, dtype=np.uint8), kernel)

    return 1 - crescent


def outerMask(center, outer_radius, border):

    # Define indices relative to center
    y, x = np.indices((512, 512))
    y = y - center[0]
    x = x - center[1]

    # Define mask by distance to center
    mask = np.sqrt(y ** 2 + x ** 2) < (outer_radius + border)

    return mask


def cloudMask(center, area, radius, angle):

    # Cloud mask is a combination of crescent and outer mask with fixed borders
    mask = crescentMask(center, area, radius, angle, 35) * outerMask(center, radius, 20)
    return mask
